#!/usr/bin/python3

from odcs.client.odcs import ODCS, AuthMech, ComposeSourceRawConfig
import fedora_messaging.api
import fedora_messaging.config
import koji
import logging
import os

# Set local logging
logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)
fedora_messaging.config.conf.load_config("/etc/fedora-messaging/fedora-minimal-compose.toml")


class Consumer(object):
    """
    A fedora-messaging consumer that process messages
    """

    def __init__(self):
        """
        Configure the consumer
        """
        self.config = fedora_messaging.config.conf["consumer_config"]
        self.rawhide = self.config['rawhide']
        self.fmc_tag = f'f{self.rawhide}-minimal-compose'
        self.odcs_url = self.config['odcs_url']
        self.odcs_file = self.config['odcs_file']
        self.packages_list = self.config['packages_list']
        # ['anaconda', 'authconfig', 'python-blivet', 'pyparted', 'parted', 'pykickstart', 'blivet-gui', 'libblockdev', 'e2fsprogs', 'dosfstools', 'grub2', 'shim-signed', 'libselinux']
        # When we have the ability to run one compose for multiple builds, use the following list
        # packages_list = ['anaconda', 'authconfig', 'python-blivet', 'pyparted', 'parted', 'pykickstart', 'blivet-gui', 'libblockdev', 'e2fsprogs', 'dosfstools', 'grub2', 'shim-signed', 'libselinux', 'chrony', 'fcoe-utils', 'hfsplus-tools', 'firewalld', 'realmd', 'yelp', 'libtimezonemap', 'kernel', 'systemd', 'dracut', 'plymouth', 'gtk3', 'python3']
        # Create a koji session
        self.kojisession = koji.ClientSession(self.config['koji_url'])
        self.kojisession.gssapi_login(principal=self.config['principal'], keytab=self.config['keytab'])
        self.kojisession.multicall = True

    def save_nvr_to_file(self, nvr):
        with open('builds_compose.log', 'w+') as f:
            logger.info(f'Build that triggered the compose:{nvr}')
            f.write(nvr + '\n')

    def run_koji_tasks(self, new_build):
        # koji untag current builds
        old_builds = self.kojisession.listTagged(self.fmc_tag)
        logger.info(f'Untagging old builds from {self.fmc_tag} : {old_builds}')
        for old_build in old_builds:
            try:
                self.kojisession.untagBuild(self.fmc_tag, old_build)
            except Exception as e:
                logger.error(f'Untagging {old_build} from {self.fmc_tag} failed, exception: {e}')
        self.kojisession.multiCall()
        # tag the new build
        logger.info(f'Tagging the koji build {new_build} to {self.fmc_tag} tag')
        try:
            self.kojisession.tagBuild(self.fmc_tag, new_build)
        except Exception as e:
            logger.error(f'Tagging {new_build} from {self.fmc_tag} failed, exception: {e}')
        self.kojisession.multiCall()

    def get_odcs_token(self, odcs_file):
        if os.path.exists(odcs_file):
            f = open(odcs_file, "r")
            token = f.readline()
            return token
        else:
            logger.error('ODCS token file is missing.')

    def run_compose(self, odcs_token):
        odcs = ODCS(self.odcs_url,
                    auth_mech=AuthMech.OpenIDC,
                    openidc_token=odcs_token)
        source = ComposeSourceRawConfig("releng_fmc", "master")

        compose_info = odcs.request_compose(source)
        logger.info(compose_info)
        return compose_info

    def inject_builds_compose(self, compose_info):
        compose_id = compose_info.get('id')
        os.rename('builds_compose.log', f'/srv/odcs/odcs-{compose_id}/logs/builds_compose.log')

    def __call__(self, message: fedora_messaging.api.Message):
        """
        Invoked when a message is received by the consumer.
        """
        logger.debug(message.topic)
        logger.debug(message.body)
        build = message.body['build']['nvr']
        package = message.body['build']['name']
        # Check if the package is the one of the packages we are interested in
        if package in self.packages_list:
            # Check if its rawhide only build
            # TODO: Extend it to branched.
            # Get the dist tag of the build
            dist_tag = build[-4:]

            if f'fc{self.rawhide}' in dist_tag:
                self.save_nvr_to_file(build)
                # Pre compose koji tasks
                self.run_koji_tasks(build)
                # Get ODCS token
                odcs_token = self.get_odcs_token(self.odcs_file)
                # Run the compose
                compose_info = self.run_compose(odcs_token)
                # Inject the file to into the compose log dir
                self.inject_builds_compose(compose_info)
